function theta_o = gllim_inverse_parameters(theta,verb)
%%%%%%%%%%%%%%%%% Inverse Mapping from Gllim Parameters %%%%%%%%%%%%%%%%%%%
%%%% Author: Antoine Deleforge (July 2012) - antoine.deleforge@inria.fr %%%
% Description: Map N observations y using the inverse conditional
% expectation E[x|y;theta] of the gllim model with parameters theta.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Input %%%%
% - y (DxN)               % Input observations to map
% - theta  (struct)       % Gllim model parameters
%   - theta.c (LxK)       % Gaussian means of X's prior
%   - theta.Gamma (LxLxK) % Gaussian covariances of X's prior
%   - theta.pi (1xK)      % Gaussian weights of X's prior
%   - theta.A (DxLxK)     % Affine transformation matrices
%   - theta.b (DxK)       % Affine transformation vectors
%   - theta.Sigma (DxDxK) % Error covariances
% - verb {0,1,2}          % Verbosity (def 1)
%%%% Output %%%%
% - theta.c (LxK)       % Gaussian means of X's prior
%   - theta.Gamma (LxLxK) % Gaussian covariances of X's prior
%   - theta.pi (1xK)      % Gaussian weights of X's prior
%   - theta.A (DxLxK)     % Affine transformation matrices
%   - theta.b (DxK)       % Affine transformation vectors
%   - theta.Sigma (DxDxK) % Error covariances
%   - theta.As (LxDxK)    % Affine transformation inverse matrices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[D,~,~]=size(theta.A);
[L,K]=size(theta.c);
% ======================Inverse density parameters=========================
if(verb>=1);fprintf(1,'Compute K projections to X space and weights\n');end;
% Parameters to estimate:
% proj=NaN(L,N,K);     % K projections to X space
% logalpha=zeros(N,K); % Conditional log-weights log(p(Z=k|y;theta))
for k=1:K
    if(verb>=2), fprintf(1,'  k=%d ',k); end
    if(verb>=2), fprintf(1,'AbcG '); end
    Ak=reshape(theta.A(:,:,k),D,L); % DxL
    bk=reshape(theta.b(:,k),D,1); % Dx1
    Sigmak=reshape(theta.Sigma(:,:,k),D,D); %DxD
    ck=reshape(theta.c(:,k),L,1); % Lx1
    Gammak=reshape(theta.Gamma(:,:,k),L,L); % LxL 
    
    if(verb>=2), fprintf(1,'cks '); end
    cs(:,k)=Ak*ck+bk;
    
    if(verb>=2), fprintf(1,'Gks '); end
    Gammas(:,:,k)=Sigmak+Ak*Gammak*Ak';

    if(verb>=2), fprintf(1,'iSks '); end
    invSigmaks2=eye(L)+Gammak*Ak'/Sigmak*Ak; %Sigmaks=invSigmaks2\Gammak
    Sigmas(:,:,k)=invSigmaks2\Gammak;	
    if(verb>=2), fprintf(1,'Aks '); end
    As(:,:,k)=invSigmaks2\Gammak*Ak'/Sigmak;     

    if(verb>=2), fprintf(1,'bks '); end
    bs(:,k)=invSigmaks2\(ck-Gammak*Ak'/Sigmak*bk);   
    
    if(verb>=2), fprintf(1,'\n'); end 
end
theta_o.c = theta.c;
theta_o.cs = cs;
theta_o.Gamma = theta.Gamma;
theta_o.Gammas = Gammas;
theta_o.A = theta.A;
theta_o.As = As;
theta_o.b  = theta.b;
theta_o.bs =  bs; 
theta_o.Sigma  = theta.Sigma;
theta_o.Sigmas  = Sigmas;
theta_o.pi = theta.pi;
end
