% $Author: Vincent Drouard $    $Date: Apr 1, 2016$    $Revision: 1.0 $
%                               
% Copyright: Perception Team, INRIA-Grenoble
% Email: vincent.drouard@inria.fr

% The Head Pose algorithm is described in this paper:      
%  (1) V. Drouard, S. Ba, G. Evangelidis, A. Deleforge, and R. Horaud ,  "Head Pose Estimation via Probabilistic High-Dimensional Regression", IEEE International Conference on Image Processing (ICIP), 2015.
function displayEstimationPrima(im,pose2D,position2D,ratio)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: Display the ground truth and the estimated head pose  
%%%%%
%%% INPUT
% im: the global image
% pose2D: 2x2 matrix, first column ground truth head pose, second one
% estimated one
% position2D: 2x2 matrix, 2D position of the face in the image (center of the face box)
% ratio: coefficient to change size of the displayed arrow
%%%%%%%%%%%%%%%%%%%%%%%%%
    figure;hold on;
    [x,z] = meshgrid(0:0.1:1*size(im,2)/size(im,1),0:0.1:1);
    y = zeros(size(x));
    surface(-x,-y,-z,im,'FaceColor','texturemap','EdgeColor','none','CDataMapping','direct');
    
    for i=1:size(pose2D,2)
        % pitch
        Rx=[1 0 0;0 cosd(pose2D(1,i)) -sind(pose2D(1,i)); 0 sind(pose2D(1,i)) cosd(pose2D(1,i))];
        % roll
        Ry=eye(3);
        % yaw
        Rz=[cosd(pose2D(2,i)) sind(pose2D(2,i)) 0; -sind(pose2D(2,i)) cosd(pose2D(2,i)) 0; 0 0 1];

        % position in the image plane
        o=[-max(x(:))*position2D(1,i)/size(im,2);0;-position2D(2,i)/size(im,1)];
        C = ratio*[0 -.5 0 0; 0 0 .5 0; 0 0 0 1]; % position of the arrow

        % head pose rotation
        poseV=Rz*Ry*Rx*C;
        % translation to face position in the image plane
        e=bsxfun(@plus,o,poseV)';

        % head pose arrow
        if i ==1 % ground truth head pose
            mArrow3(e(1,:),e(3,:),'color',[0 0.7 0],'stemWidth',0.015,'tipWidth',0.02);hold on;
        else  % estimated head pose
            mArrow3(e(1,:),e(3,:),'color',[0.7 0 0],'stemWidth',0.015,'tipWidth',0.02);hold on;
        end
    end
    
    % figure property 
    axis off;
    grid off;
    view(180,0);

end
