% $Author: Vincent Drouard $    $Date: Apr 1, 2016$    $Revision: 1.0 $
%                               
% Copyright: Perception Team, INRIA-Grenoble
% Email: vincent.drouard@inria.fr

% The Head Pose algorithm is described in this paper:      
%  (1) V. Drouard, S. Ba, G. Evangelidis, A. Deleforge, and R. Horaud ,  "Head Pose Estimation via Probabilistic High-Dimensional Regression", IEEE International Conference on Image Processing (ICIP), 2015.
function [estimation,error] = estimatePrima(data,theta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: Estimate the head pose on the Prima Head Pose database
%%%%%
%%% INPUT
% data: struct containing all the process database
% theta: 15x1 cell containing struct containing model parameters for each
% person of the database
%%% OUTPUT
% estimation: 15x1 cell containing the results of the estimation
% error: 15x1 cell containing the absolute error of the estimation
%%%%%%%%%%%%%%%%%%%%%%%%%    
    for i = 1:15
        idx = find(data.index(2,:)==i);
        results = gllim_inverse_map(data.features(2:end,idx),theta{i},1);% estimation given new observations
        estimation{i} = results;
        error{i} = abs(results-data.pose(2:end,idx));%compute absolute error 
    end
end
