% $Author: Vincent Drouard $    $Date: Apr 1, 2016$    $Revision: 1.0 $
%                               
% Copyright: Perception Team, INRIA-Grenoble
% Email: vincent.drouard@inria.fr

% The Head Pose algorithm is described in this paper:      
%  (1) V. Drouard, S. Ba, G. Evangelidis, A. Deleforge, and R. Horaud ,  "Head Pose Estimation via Probabilistic High-Dimensional Regression", IEEE International Conference on Image Processing (ICIP), 2015.

function data = processPrima(data_path)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: Process the Prima Head Pose database
%%%%%
%%% INPUT
% data_path = string containing path to find the data
%%% OUTPUT
% data = structure containing the processed data
%     - data.features: 1889xN matrix, each column [sample_index pHog_features]^T
%     - data.pose: 3xN matrix, each column [sample_index pitch yaw]^T
%     - data.box: 5xN matrix, each column [sample_index top_left_x top_left_y bottom_right_x bottom_right_x]^T
%     - data.index: 2xN matrix, each column [sample_index person_index]^T
%     - data.image_path: 2xN cells, {sample_index}{full path of the associated image}
%
%%%%%%%%%%%%%%%%%%%%%%%%%

    sample_index = 1;
    
    for f = 1:15 % loop over all the person in the database
        fprintf('ID %02d\n',f);
        images = dir([data_path 'data/Person' sprintf('%02d',f) '/*.jpg']);% list all images of a person
        reverseStr = [];
        for i = 1:length(images)% loop over all images of a person
            % read image
            frame = imread([data_path 'data/Person' sprintf('%02d',f) '/' images(i).name]);
            % read associated text file with head pose information and ground truth annotated bounding box value
            id = fopen([data_path 'data/Person' sprintf('%02d',f) '/' images(i).name(1:end-3) 'txt']);
            st=fread(id,'*char')';
            fclose(id);
            % extract booding box related information
            tmp = str2num(st(28:end-1));
            bbox = [tmp(1)-tmp(3)/2;tmp(2)-tmp(4)/1.5;tmp(1)+tmp(3)/2;tmp(2)+tmp(4)/1.5];
            bbox(1) = max(1,bbox(1));bbox(2) = max(1,bbox(2));bbox(3) = min(size(frame,2),bbox(3));bbox(4) = min(size(frame,1),bbox(4));
            face = frame(round(bbox(2)):round(bbox(4)),...
                round(bbox(1)):round(bbox(3)),:);% extract face from image
            face = histeq(rgb2gray(imresize(face,[64 64])));% resize and convert to grayscale
            hog = [HOG(face,8,[32 32],[2 2]);...
                HOG(face,8,[16 16],[2 2]);...
                HOG(face,8,[8 8],[2 2])];% compute feature vector
            data.features(:,sample_index)=[sample_index;hog];
            % extract head pose value from image name
            p=images(i).name(12:end-4);
            a=find(p=='+' | p=='-');
            data.pose(:,sample_index) = [sample_index;str2double(p(a(1):a(2)-1));str2double(p(a(2):end))];

            % save information
            data.box(:,sample_index) = [sample_index;bbox];
            data.index(:,sample_index) = [sample_index;f];
            data.image_path(:,sample_index) = {sample_index,[data_path 'data/Person' sprintf('%02d',f) '/' images(i).name]};

            sample_index = sample_index +1;
            % display
            msg = sprintf('Processed %d/%d', i, length(images));
            fprintf([reverseStr, msg]);
            reverseStr = repmat(sprintf('\b'), 1, length(msg));
        end
        fprintf('\n');
    end
end
